.light1 {
  color: white;
  margin: 1px;
}
.light2 {
  color: black;
  margin: 1px;
}
.max1 {
  width: 6;
}
.max2 {
  width: 8;
}
.glob1 {
  margin: auto auto;
}
.ops1 {
  height: gt-or-eq;
  height: lt-or-eq;
  height: lt-or-eq-alias;
}
.ops2 {
  height: gt-or-eq;
  height: not-eq;
}
.ops3 {
  height: lt-or-eq;
  height: lt-or-eq-alias;
  height: not-eq;
}
.default1 {
  content: default;
}
.test1 {
  content: "true.";
}
.test2 {
  content: "false.";
}
.test3 {
  content: "false.";
}
.test4 {
  content: "false.";
}
.test5 {
  content: "false.";
}
.bool1 {
  content: true and true;
  content: true;
  content: false, true;
  content: false and true and true, true;
  content: false, true and true;
  content: false, false, true;
  content: false, true and true and true, false;
  content: not false;
  content: not false and false, not false;
}
.equality-units {
  test: pass;
}
.colorguardtest {
  content: is #ff0000;
  content: is not #0000ff its #ff0000;
  content: is not #0000ff its #800080;
}
.stringguardtest {
  content: is theme1;
  content: is not theme2;
  content: is theme1 no quotes;
}
#tryNumberPx {
  catch: all;
  declare: 4;
  declare: 4px;
}
.call-lock-mixin .call-inner-lock-mixin {
  a: 1;
  x: 1;
}
