.ext1 .ext2,
.all .ext2 {
  background: black;
}
@media tv {
  .ext1 .ext3,
  .tv-lowres .ext3,
  .all .ext3 {
    color: white;
  }
  .tv-lowres {
    background: blue;
  }
}
@media tv and hires {
  .ext1 .ext4,
  .tv-hires .ext4,
  .all .ext4 {
    color: green;
  }
  .tv-hires {
    background: red;
  }
}
