.clearfix {
  *zoom: 1;
  &:after {
    content: '';
    display: block;
    clear: both;
    height: 0;
  }
}

.foo {
  &:extend(.clearfix all);
  color: red;
}

.bar {
  &:extend(.clearfix all);
  color: blue;
}
