.replace.replace .replace,
.c.replace + .replace .replace,
.replace.replace .c,
.c.replace + .replace .c,
.rep_ace {
  prop: copy-paste-replace;
}
.a .b .c {
  prop: not_effected;
}
.a,
.effected {
  prop: is_effected;
}
.a .b {
  prop: not_effected;
}
.a .b.c {
  prop: not_effected;
}
.c .b .a,
.a .b .a,
.c .a .a,
.a .a .a,
.c .b .c,
.a .b .c,
.c .a .c,
.a .a .c {
  prop: not_effected;
}
.e.e,
.dbl {
  prop: extend-double;
}
.e.e:hover {
  hover: not-extended;
}
