#header {
  color: (@base-color * 3);
  border-left: @the-border;
  border-right: (@the-border * 2);
}
#footer {
  color: (@base-color + #003300);
  border-color: @red;
}
@red: blue; // var is overridden by the modifyVars
//@base-color: green;