.mix-inner (@var) {
  border-width: @var;
}

.mix (@a: 10) {
  .inner {
    height: (@a * 10);

    .innest {
      width: @a;  
      .mix-inner((@a * 2));
    }  
  }
}

.class {
  .mix(30);
}

.class2 {
  .mix(60);
}
