@import "../css/background.css";
@import "import-test-d.css";

@import "imports/logo";
@import "imports/font";

.unquoted-relative-path-bg() {
    background-image: url(../../data/image.jpg);
}
.quoted-relative-path-border-image() {
    border-image: url('../../data/image.jpg');
}

#imported-relative-path {
    .unquoted-relative-path-bg;
    .quoted-relative-path-border-image;
}