@charset "UTF-8";

@import "import/test.less";

.start() {
  .test2 {
      color: red;
  }
}

.mix() {
    color: black;
}

.test1 {
  .mix();
}

.start();

.mixin_import1();

.mixin_import2();

@debug: 1;
& when (@debug = 1) {
  .test {
    color: red;
    & when (@debug = 1) {
      width: 2;
    }
  }
}