.eval {
  js: 42;
  js: 2;
  js: "hello world";
  js: 1, 2, 3;
  title: "string";
  ternary: true;
  multiline: 2;
}
.scope {
  var: 42;
  escaped: 7px;
}
.vars {
  width: 8;
}
.escape-interpol {
  width: hello world;
}
.arrays {
  ary: "1, 2, 3";
  ary1: "1, 2, 3";
}
