/******************\
*                  *
*  Comment Header  *
*                  *
\******************/
/*

    Comment

*/
/*
 * Comment Test
 *
 * - cloudhead (http://cloudhead.net)
 *
 */
/* Colors
 * ------
 *   #EDF8FC (background blue)
 *   #166C89 (darkest blue)
 *
 * Text:
 *   #333 (standard text) // A comment within a comment!
 *   #1F9EC9 (standard link)
 *
 */
/* @group Variables
------------------- */
#comments,
.comments {
  /**/
  color: red;
  /* A C-style comment */
  /* A C-style comment */
  background-color: orange;
  font-size: 12px;
  /* lost comment */
  content: "content";
  border: 1px solid black;
  padding: 0;
  margin: 2em;
}
/* commented out
  #more-comments {
    color: grey;
  }
*/
.selector,
.lots,
.comments {
  color: #808080, /* blue */ #ffa500;
  -webkit-border-radius: 2px /* webkit only */;
  -moz-border-radius: 8px /* moz only with operation */;
}
.test {
  color: 1px;
}
#last {
  color: #0000ff;
}
/*  */
/* { */
/*  */
/*  */
/*  */
#div {
  color: #A33;
}
/* } */
