.parens {
  border: 2px solid #000000;
  margin: 1px 3px 16 3;
  width: 36;
  padding: 2px 36px;
}
.more-parens {
  padding: 8 4 4 4px;
  width-all: 96;
  width-first: 16 * 6;
  width-keep: (4 * 4) * 6;
  height-keep: (7 * 7) + (8 * 8);
  height-all: 113;
  height-parts: 49 + 64;
  margin-keep: (4 * (5 + 5) / 2) - (4 * 2);
  margin-parts: 20 - 8;
  margin-all: 12;
  border-radius-keep: 4px * (1 + 1) / 4 + 3px;
  border-radius-parts: 8px / 7px;
  border-radius-all: 5px;
}
.negative {
  neg-var: -1;
  neg-var-paren: -(1);
}
.nested-parens {
  width: 2 * (4 * (2 + (1 + 6))) - 1;
  height: ((2 + 3) * (2 + 3) / (9 - 4)) + 1;
}
.mixed-units {
  margin: 2px 4em 1 5pc;
  padding: 6px 1em 2px 2;
}
.test-false-negatives {
  a: (;
}
