/*
  The media statement above is invalid (no selector)
  We should ban invalid media queries with properties and no selector?
*/
.visible {
  color: red;
}
.visible .c {
  color: green;
}
.visible {
  color: green;
}
.visible:hover {
  color: green;
}
.only-with-visible + .visible,
.visible + .only-with-visible,
.visible + .visible {
  color: green;
}
.only-with-visible + .visible .sub,
.visible + .only-with-visible .sub,
.visible + .visible .sub {
  color: green;
}
.b {
  color: red;
  color: green;
}
.b .c {
  color: green;
}
.b:hover {
  color: green;
}
.b + .b {
  color: green;
}
.b + .b .sub {
  color: green;
}
.y {
  pulled-in: yes;
}
/* comment pulled in */
.visible {
  extend: test;
}
