.foo {
  a: 1;
}
.foo {
  a: 2;
}
#foo {
  a: 3;
}
#foo {
  a: 4;
}
mi-test-a {
  a: 1;
  a: 2;
  a: 3;
  a: 4;
}
.b .bb.foo-xxx .yyy-foo#foo .foo.bbb {
  b: 1;
}
mi-test-b {
  b: 1;
}
#foo-foo > .bar .baz {
  c: c;
}
mi-test-c-1 > .bar .baz {
  c: c;
}
mi-test-c-2 .baz {
  c: c;
}
mi-test-c-3 {
  c: c;
}
mi-test-d {
  gender: "Male";
}
