.test1 {
  transform: rotate(90deg), skew(30deg), scale(2, 4);
}
.test2 {
  transform: rotate(90deg), skew(30deg);
  transform: scaleX(45deg);
}
.test3 {
  transform: scaleX(45deg);
  background: url(data://img1.png);
}
.test4 {
  transform: rotate(90deg), skew(30deg);
  transform: scale(2, 4) !important;
}
.test5 {
  transform: rotate(90deg), skew(30deg);
  transform: scale(2, 4) !important;
}
.test6 {
  transform: scale(2, 4);
}
.test-interleaved {
  transform: t1, t2, t3;
  background: b1, b2, b3;
}
