#header {
  color: (@base-color * 3);
  border-left: @the-border;
  border-right: (@the-border * 2);
}
#footer {
  color: (@base-color + #003300);
  border-color: @red;
}
@red: desaturate(red, 10%); // less file overrides passed in color <- note line comment on last line to check it is okay