@import url(http://fonts.googleapis.com/css?family=Open+Sans);
@import url(/absolute/something.css) screen and (color) and (max-width: 600px);
@import url("//ha.com/file.css") (min-width: 100px);
#import-test {
  height: 10px;
  color: #ff0000;
  width: 10px;
  height: 30%;
}
@media screen and (max-width: 600px) {
  body {
    width: 100%;
  }
}
#import {
  color: #ff0000;
}
.mixin {
  height: 10px;
  color: #ff0000;
}
@media screen and (max-width: 601px) {
  #css {
    color: yellow;
  }
}
@media screen and (max-width: 602px) {
  body {
    width: 100%;
  }
}
@media screen and (max-width: 603px) {
  #css {
    color: yellow;
  }
}
