@import "import/import-once-test-c";
@import "import/import-once-test-c";
@import "import/import-once-test-c.less";
@import "import/deeper/import-once-test-a";
@import (multiple) "import/import-test-f.less";
@import (multiple) "import/import-test-f.less";