.light {
  color: green;
}
.see-the {
  color: green;
}
.hide-the {
  color: green;
}
.multiple-conditions-1 {
  color: red;
}
.inheritance .test {
  color: black;
}
.inheritance:hover {
  color: pink;
}
.clsWithGuard {
  dispaly: none;
}
.dont-split-me-up {
  width: 1px;
  color: red;
  height: 1px;
}
 + .dont-split-me-up {
  sibling: true;
}
.scope-check {
  sub-prop: 2px;
  prop: 1px;
}
.scope-check-2 {
  sub-prop: 2px;
  prop: 1px;
}
