.error,
.badError {
  border: 1px #f00;
  background: #fdd;
}
.error.intrusion,
.badError.intrusion {
  font-size: 1.3em;
  font-weight: bold;
}
.intrusion .error,
.intrusion .badError {
  display: none;
}
.badError {
  border-width: 3px;
}
.foo .bar,
.foo .baz,
.ext1 .ext2 .bar,
.ext1 .ext2 .baz,
.ext3 .bar,
.ext3 .baz,
.foo .ext3,
.ext4 .bar,
.ext4 .baz,
.foo .ext4 {
  display: none;
}
div.ext5,
.ext6 > .ext5,
div.ext7,
.ext6 > .ext7 {
  width: 100px;
}
.ext8.ext9,
.fuu {
  result: add-foo;
}
.ext8 .ext9,
.ext8 + .ext9,
.ext8 > .ext9,
.buu,
.zap,
.zoo {
  result: bar-matched;
}
.ext8.nomatch {
  result: none;
}
.ext8 .ext9,
.buu {
  result: match-nested-bar;
}
.ext8.ext9,
.fuu {
  result: match-nested-foo;
}
.aa,
.cc {
  color: black;
}
.aa .dd,
.aa .ee {
  background: red;
}
.bb,
.cc,
.ee,
.ff {
  background: red;
}
.bb .bb,
.ff .ff {
  color: black;
}
