

.whitespace
  { color: white; }  
  
.whitespace
{
  color: white;
}
  .whitespace
{ color: white; }

.whitespace{color:white;}  
.whitespace { color : white ; }

.white,
.space,
.mania
{ color: white; }

.no-semi-column { color: white }
.no-semi-column {
  color: white;
  white-space: pre
}
.no-semi-column {border: 2px solid white}
.newlines {
  background: the,
              great,
              wall;
  border: 2px
          solid
          black;
}
.empty {
  
}
.sel
.newline_ws	.tab_ws	{
color:
white;
background-position: 45
-23;
}
