.escape\|random\|char {
  color: red;
}
.mixin\!tUp {
  font-weight: bold;
}
.\34 04 {
  background: red;
}
.\34 04 strong {
  color: #ff00ff;
  font-weight: bold;
}
.trailingTest\+ {
  color: red;
}
/* This hideous test of hideousness checks for the selector "blockquote" with various permutations of hex escapes */
\62\6c\6f \63 \6B \0071 \000075o\74 e {
  color: silver;
}
[ng\:cloak],
ng\:form {
  display: none;
}
