#colours {
  color1: #fea;
  color2: #ffeeaa;
  color3: rgba(255, 238, 170, 0.1);
  @color1: #fea;
  string: "@{color1}";
  /* comments are stripped */
  // both types!
  /*! but not this type
      Note preserved whitespace
  */
}
dimensions {
  val: 0.1px;
  val: 0em;
  val: 4cm;
  val: 0.2;
  val: 5;
  angles-must-have-unit: 0deg;
  durations-must-have-unit: 0s;
  length-doesnt-have-unit: 0px;
  width: auto\9;
}
@page  {
  marks: none;
@top-left-corner {
  vertical-align: top;
}
@top-left {
  vertical-align: top;
}
}
.shadow ^ .dom,
body ^^ .shadow {
  display: done;
}
